OPTIONS nonumber nodate nocenter pagesize=MAX linesize=120 formdlim='-';											
* Kill default titles on output; 
TITLE;
*TURN OFF HTML OUTPUT;
ODS HTML CLOSE;
*TURN ON LISTING OUTPUT;
ODS LISTING;


* CHANGE THIS TO YOUR FOLDER:;
%LET filepath = C:\Users\jtemplin2\Dropbox\!PSYC 943\Lectures\11 More Matrix Algebra, Mean Vectors, Covariance Matrices, and MVN\SAS Example;

*NAME A PERMANANT LIBRARY FOR SAS DATA;
LIBNAME SAT "&filepath."; 

*IMPORT SAT DATA (COMMA DELIMITED) INTO SAS LIBRARY;
DATA SAT.satdata;
    INFILE "&filepath.\sat.csv"
		DLM = ',' DSD FIRSTOBS=2;
    INPUT satv satm;
RUN;

*COPY SAT DATA TO TEMPORARY (WORK) DIRECTORY;
DATA work.satdata;
	SET SAT.satdata;
RUN;

PROC IML;
RESET PRINT; *THIS LINE PRINTS THE RESULTS FROM ALL MATRIX EQUATIONS;

*READ IN DATA FROM DATA SET AND PUT INTO MATRIX X;
USE work.satdata;
READ ALL VAR _ALL_ INTO X;

*NUMBER OF OBSERVATIONS IN X:;
N = NROW(X);

*MATRIX MULTIPLICATION SAT EXAMPLE;
XTX = t(X)*X;
XTXinv = inv(XTX);

*ONES VECTOR WITH SAME LENGTH AS NUMBER OF OBSERVATIONS;
ONES = J(N,1,1); *J function (built in) creates a new matrix with (#rows, #cols, value of element);

*CALCULATION OF THE MEAN VECTOR;
meanvec = (1/N)*t(X)*ONES;

*CALCULATION OF THE COVARIANCE MATRIX;
mean_matrix = ONES*t(meanvec); *for covariance matrix;
cov_matrix = (1/N)*t(X - mean_matrix)*(X - mean_matrix);

*DIAGONAL MATRIX OF STANDARD DEVIATIONS FROM COVARIANCE MATRIX:;
*SQRT TAKES STANDARD DEVIATION (COVARIANCE MATRIX HAS VARIANCES);
D_matrix = SQRT(DIAG(cov_matrix));

*INVERSE OF D_Matrix:;
D_matrix_inv = INV(D_matrix);

*CORRELATION MATRIX:;
corr_matrix = D_matrix_inv*cov_matrix*D_matrix_inv;

*GENERALIZED VARIANCE:;
GEN_VAR = DET(cov_matrix);

*TOTAL SAMPLE VARIANCE;
TOT_VAR = TRACE(cov_matrix);


*MULTIVARIATE NORMAL DISTRIBUTION FUNCTION CALCULATIONS;
*CONSTANTS FOR ALL CALCULATIONS:;

PI = CONSTANT('pi'); *the constant pi;
NVAR = NCOL(X); *the number of variables in X;

pi_constant = (2*PI)**(NVAR/2);
sigma_constant = DET(cov_matrix)**(1/2);
sigma_inverse = INV(cov_matrix);


*OBSERVATION #632:;
obs = X[631,];
mean_diff = t(obs)-meanvec;
exponent_term = (-1/2)*t(mean_diff)*sigma_inverse*mean_diff;
likelihood = (1/pi_constant)*(1/sigma_constant)*exp(exponent_term);

*OBSERVATION #717:;
obs = X[717,];
mean_diff = t(obs)-meanvec;
exponent_term = (-1/2)*t(mean_diff)*sigma_inverse*mean_diff;
likelihood = (1/pi_constant)*(1/sigma_constant)*exp(exponent_term);

*likelihood at mean:;
obs = t(meanvec);
mean_diff = t(obs)-meanvec;
exponent_term = (-1/2)*t(mean_diff)*sigma_inverse*mean_diff;
likelihood = (1/pi_constant)*(1/sigma_constant)*exp(exponent_term);


QUIT;



